(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright (c) 2013-2014 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uindex_universe.mli,v 1.6 2014/05/06 12:56:18 fclement Exp $*)

(** {3 Numbering indices} *)

type 'a element = 'a Uindex_hashtbl.key;; (* = 'a Uindex.t *)
type 'a index = 'a Uindex_hashtbl.value;; (* = 'a element Uindex.t *)

type 'a t = private
  {
   tables : 'a Uindex_hashtbl.t;
   cardinal : Upervasives.length;
  }
;;

val equal : 'a t -> 'a t -> bool;;
(** Structural equality of index universes. *)

(** {6 Construction functions} *)

val of_nonempty_range : 'a element -> 'a element -> 'a t;;
(** The first element must be lesser than the second. *)

val of_array : 'a element array -> 'a t;;
val of_list1 : 'a element Ulist1.t -> 'a t;;

(** {6 Access functions} *)

val cardinal : 'a t -> Upervasives.length;;

val mem : 'a t -> 'a element -> bool;;

val index : 'a t -> 'a element -> 'a index;;

val element : 'a t -> 'a index -> 'a element;;
val elements : 'a t -> 'a element array;;

(** {6 Iterators} *)

val iter : ('a index -> 'a element -> unit) -> 'a t -> unit;;
val sub_iter :
  ('a index -> 'a element -> unit) ->
  'a t -> 'a index -> Upervasives.length -> unit
;;

val fold_left :
  ('accu -> 'a index -> 'a element -> 'accu) -> 'accu -> 'a t -> 'accu
;;
